#!/usr/bin/env python

"""
Common scheduling functionality.

Copyright (C) 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.dates import ValidityError

def schedule_in_freebusy(handler):

    """
    Attempt to schedule the current object of the given 'handler' in the
    free/busy schedule of a resource, returning an indication of the kind of
    response to be returned.
    """

    # If newer than any old version, discard old details from the
    # free/busy record and check for suitability.

    periods = handler.get_periods(handler.obj)

    freebusy = handler.store.get_freebusy(handler.user)
    offers = handler.store.get_freebusy_offers(handler.user)

    # Check the periods against any scheduled events and against
    # any outstanding offers.

    scheduled = handler.can_schedule(freebusy, periods)
    scheduled = scheduled and handler.can_schedule(offers, periods)

    return scheduled and "ACCEPTED" or "DECLINED"

def schedule_corrected_in_freebusy(handler):

    """
    Attempt to schedule the current object of the given 'handler', correcting
    specified datetimes according to the configuration of a resource,
    returning an indication of the kind of response to be returned.
    """

    obj = handler.obj.copy()

    # Check any constraints on the request.

    try:
        corrected = handler.correct_object()

    # Refuse to schedule obviously invalid requests.

    except ValidityError:
        return None

    # With a valid request, determine whether the event can be scheduled.

    scheduled = schedule_in_freebusy(handler)

    # Restore the original object if it was corrected but could not be
    # scheduled.

    if scheduled == "DECLINED" and corrected:
        handler.set_object(obj)
    
    # Where the corrected object can be scheduled, issue a counter
    # request.

    return scheduled == "ACCEPTED" and (corrected and "COUNTER" or "ACCEPTED") or "DECLINED"

# Registry of scheduling functions.

scheduling_functions = {
    "schedule_in_freebusy" : schedule_in_freebusy,
    "schedule_corrected_in_freebusy" : schedule_corrected_in_freebusy,
    }

# vim: tabstop=4 expandtab shiftwidth=4
